@@ -20,6 +20,7 @@
 #include <Python.h>
 #include <apr_general.h>
 #include <svn_fs.h>
+#include <svn_path.h>
 #include <svn_repos.h>
 #include <apr_md5.h>
 
@@ -30,13 +31,6 @@ extern PyTypeObject Repository_Type;
 extern PyTypeObject FileSystem_Type;
 extern PyTypeObject Stream_Type;
 
-typedef struct {
-	PyObject_HEAD
-	svn_stream_t *stream;
-	apr_pool_t *pool;
-	svn_boolean_t closed;
-} StreamObject;
-
 typedef struct { 
 	PyObject_HEAD
     apr_pool_t *pool;
@@ -66,7 +60,8 @@ static PyObject *repos_create(PyObject *self, PyObject *args)
 		PyErr_SetString(PyExc_RuntimeError, "Unable to create fs config hash");
 		return NULL;
 	}
-    RUN_SVN_WITH_POOL(pool, svn_repos_create(&repos, path, NULL, NULL, 
+    RUN_SVN_WITH_POOL(pool, svn_repos_create(&repos,
+                svn_path_canonicalize(path, pool), NULL, NULL,
                 hash_config, hash_fs_config, pool));
 
 	ret = PyObject_New(RepositoryObject, &Repository_Type);
@@ -105,7 +100,8 @@ static PyObject *repos_init(PyTypeObject *type, PyObject *args, PyObject *kwargs
 	if (ret->pool == NULL)
 		return NULL;
 	Py_BEGIN_ALLOW_THREADS
-	err = svn_repos_open(&ret->repos, path, ret->pool);
+	err = svn_repos_open(&ret->repos, svn_path_canonicalize(path, ret->pool),
+                            ret->pool);
 	Py_END_ALLOW_THREADS
     if (!check_error(err)) {
 		Py_DECREF(ret);
@@ -769,184 +765,6 @@ PyTypeObject FileSystemRoot_Type = {
 };
 
 
-static void stream_dealloc(PyObject *self)
-{
-	StreamObject *streamself = (StreamObject *)self;
-
-	apr_pool_destroy(streamself->pool);
-}
-
-static PyObject *stream_init(PyTypeObject *type, PyObject *args, PyObject *kwargs)
-{
-	char *kwnames[] = { NULL };
-	StreamObject *ret;
-
-	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "", kwnames))
-		return NULL;
-
-	ret = PyObject_New(StreamObject, &Stream_Type);
-	if (ret == NULL)
-		return NULL;
-
-	ret->pool = Pool(NULL);
-	if (ret->pool == NULL)
-		return NULL;
-	ret->stream = svn_stream_empty(ret->pool);
-	ret->closed = FALSE;
-
-	return (PyObject *)ret;
-}
-
-static PyObject *stream_close(StreamObject *self)
-{
-	if (!self->closed) {
-		svn_stream_close(self->stream);
-		self->closed = TRUE;
-	}
-	Py_RETURN_NONE;
-}
-
-static PyObject *stream_write(StreamObject *self, PyObject *args)
-{
-	char *buffer;
-	size_t len;
-	if (!PyArg_ParseTuple(args, "s#", &buffer, &len))
-		return NULL;
-
-	if (self->closed) {
-		PyErr_SetString(PyExc_RuntimeError, "unable to write: stream already closed");
-		return NULL;
-	}
-
-	RUN_SVN(svn_stream_write(self->stream, buffer, &len));
-
-	return PyInt_FromLong(len);
-}
-
-static PyObject *stream_read(StreamObject *self, PyObject *args)
-{
-	PyObject *ret;
-	apr_pool_t *temp_pool;
-	long len = -1;
-	if (!PyArg_ParseTuple(args, "|l", &len))
-		return NULL;
-
-	if (self->closed) {
-		return PyString_FromString("");
-	}
-
-	temp_pool = Pool(NULL);
-	if (temp_pool == NULL) 
-		return NULL;
-	if (len != -1) {
-		char *buffer;
-		apr_size_t size = len;
-		buffer = apr_palloc(temp_pool, len);
-		if (buffer == NULL) {
-			PyErr_NoMemory();
-			apr_pool_destroy(temp_pool);
-			return NULL;
-		}
-		RUN_SVN_WITH_POOL(temp_pool, svn_stream_read(self->stream, buffer, &size));
-		ret = PyString_FromStringAndSize(buffer, size);
-		apr_pool_destroy(temp_pool);
-		return ret;
-	} else {
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 6
-		svn_string_t *result;
-		RUN_SVN_WITH_POOL(temp_pool, svn_string_from_stream(&result, 
-							   self->stream,
-							   temp_pool,
-							   temp_pool));
-		self->closed = TRUE;
-		ret = PyString_FromStringAndSize(result->data, result->len);
-		apr_pool_destroy(temp_pool);
-		return ret;
-#else
-		PyErr_SetString(PyExc_NotImplementedError, 
-			"Subversion 1.5 does not provide svn_string_from_stream().");
-		return NULL;
-#endif
-	}
-}
-
-static PyMethodDef stream_methods[] = {
-	{ "read", (PyCFunction)stream_read, METH_VARARGS, NULL },
-	{ "write", (PyCFunction)stream_write, METH_VARARGS, NULL },
-	{ "close", (PyCFunction)stream_close, METH_NOARGS, NULL },
-	{ NULL, }
-};
-
-PyTypeObject Stream_Type = {
-	PyObject_HEAD_INIT(NULL) 0,
-	"repos.Stream", /*	const char *tp_name;  For printing, in format "<module>.<name>" */
-	sizeof(StreamObject), 
-	0,/*	Py_ssize_t tp_basicsize, tp_itemsize;  For allocation */
-	
-	/* Methods to implement standard operations */
-	
-	stream_dealloc, /*	destructor tp_dealloc;	*/
-	NULL, /*	printfunc tp_print;	*/
-	NULL, /*	getattrfunc tp_getattr;	*/
-	NULL, /*	setattrfunc tp_setattr;	*/
-	NULL, /*	cmpfunc tp_compare;	*/
-	NULL, /*	reprfunc tp_repr;	*/
-	
-	/* Method suites for standard classes */
-	
-	NULL, /*	PyNumberMethods *tp_as_number;	*/
-	NULL, /*	PySequenceMethods *tp_as_sequence;	*/
-	NULL, /*	PyMappingMethods *tp_as_mapping;	*/
-	
-	/* More standard operations (here for binary compatibility) */
-	
-	NULL, /*	hashfunc tp_hash;	*/
-	NULL, /*	ternaryfunc tp_call;	*/
-	NULL, /*	reprfunc tp_str;	*/
-	NULL, /*	getattrofunc tp_getattro;	*/
-	NULL, /*	setattrofunc tp_setattro;	*/
-	
-	/* Functions to access object as input/output buffer */
-	NULL, /*	PyBufferProcs *tp_as_buffer;	*/
-	
-	/* Flags to define presence of optional/expanded features */
-	0, /*	long tp_flags;	*/
-	
-	NULL, /*	const char *tp_doc;  Documentation string */
-	
-	/* Assigned meaning in release 2.0 */
-	/* call function for all accessible objects */
-	NULL, /*	traverseproc tp_traverse;	*/
-	
-	/* delete references to contained objects */
-	NULL, /*	inquiry tp_clear;	*/
-	
-	/* Assigned meaning in release 2.1 */
-	/* rich comparisons */
-	NULL, /*	richcmpfunc tp_richcompare;	*/
-	
-	/* weak reference enabler */
-	0, /*	Py_ssize_t tp_weaklistoffset;	*/
-	
-	/* Added in release 2.2 */
-	/* Iterators */
-	NULL, /*	getiterfunc tp_iter;	*/
-	NULL, /*	iternextfunc tp_iternext;	*/
-	
-	/* Attribute descriptor and subclassing stuff */
-	stream_methods, /*	struct PyMethodDef *tp_methods;	*/
-	NULL, /*	struct PyMemberDef *tp_members;	*/
-	NULL, /*	struct PyGetSetDef *tp_getset;	*/
-	NULL, /*	struct _typeobject *tp_base;	*/
-	NULL, /*	PyObject *tp_dict;	*/
-	NULL, /*	descrgetfunc tp_descr_get;	*/
-	NULL, /*	descrsetfunc tp_descr_set;	*/
-	0, /*	Py_ssize_t tp_dictoffset;	*/
-	NULL, /*	initproc tp_init;	*/
-	NULL, /*	allocfunc tp_alloc;	*/
-	stream_init, /* tp_new tp_new */
-};
-
 void initrepos(void)
 {
 	static apr_pool_t *pool;
